'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ipcBridge = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _mobx = require('mobx');

var _cacheFn = require('./cacheFn.js');

var _hasOwnProperty = require('../util/hasOwnProperty.js');

var _myExceptions = require('../util/my-exceptions.js');

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

// bSpec=[srvMap,clientMap]  Map: fname->fnDef
// fnDef= function || {fn,client?,srv?} client/srv:ctrl
// ctrl={start,proc,stop}

// fnSpec={peerId,fname, ipcSpec?, v0?}
// ipcSpec={ipcUrl,myId?}
// cSpec={peerId,fname,args}

var noFn = function noFn() {
  return false;
};
var noCtrl = {
  client: {
    start: function start(info) {
      return info.connect(info.args);
    },
    proc: function proc(info, data, state) {
      return info.recv(data);
    },
    stop: function stop(info, state) {}
  },
  srv: {
    start: function start(info) {
      var yomo = info.yomo;
      var sendData = info.sendData;
      var fnDef = info.fnDef;
      var args = info.args;
      //++ forward exceptions; catch incorrect fnDef outside

      return (0, _cacheFn.yomoRun)(yomo, false, function () {
        return sendData((fnDef.fn || fnDef).apply(undefined, [yomo].concat(_toConsumableArray(args))));
      });
    },
    proc: function proc(info, data, state) {
      return state;
    },
    stop: function stop(info, state) {
      return state && state();
    }
  }
};

var cacheConn = function cacheConn(ipc, bSpec) {
  return (0, _cacheFn.cacheFnu)(function (yomo, ipcSpec) {
    var _ref = bSpec || [];

    var _ref2 = _slicedToArray(_ref, 2);

    var srvMap = _ref2[0];
    var clientMap = _ref2[1];

    srvMap = srvMap || {};clientMap = clientMap || srvMap;

    var lookup = function lookup(onServer, cSpec, cb) {
      var peerId = cSpec.peerId;
      var fname = cSpec.fname;
      var args = cSpec.args;

      var fnDef = (onServer ? srvMap : clientMap)[fname] || noFn;
      var k = onServer ? 'srv' : 'client';
      var ctrl = fnDef[k] || noCtrl[k];
      cb({ yomo: yomo, ctrl: ctrl, fnDef: fnDef });
    };

    return ipc(ipcSpec, lookup);
  });
};

var connBridge = (0, _cacheFn.metaFn)(function (_ref3, yomo, _ref4) {
  var _ref6 = _slicedToArray(_ref3, 2);

  var cfn = _ref6[0];
  var ipcSpec = _ref6[1];

  var _ref5 = _toArray(_ref4);

  var fnSpec = _ref5[0];

  var args = _ref5.slice(1);

  var v0 = (0, _hasOwnProperty.hasOwnProperty)(fnSpec, 'v0') ? fnSpec.v0 : _myExceptions.vWait;
  var res = (0, _mobx.observable)((0, _mobx.asReference)(v0));
  var recv = (0, _mobx.action)(function (data) {
    return res.set(data);
  });
  var cDone = void 0;
  var done = (0, _cacheFn.yomoRun)(yomo, false, function () {
    cDone && cDone();
    var spec = ipcSpec || fnSpec.ipcSpec;
    var conn = spec && cfn(yomo, spec);
    cDone = conn && conn.connectFn(_extends({}, fnSpec, { args: args }), recv);
  });
  res.unsub = function () {
    done();cDone && cDone();
  };
  return res;
});

var ipcBridge = exports.ipcBridge = function ipcBridge(ipc, bSpec, ipcSpec) {
  return connBridge(cacheConn(ipc, bSpec), ipcSpec);
};