'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mqttBridge = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _mobx = require('mobx');

var _mqtt = require('mqtt');

var _mqtt2 = _interopRequireDefault(_mqtt);

var _cacheFn = require('./cacheFn');

var _reuse = require('../util/reuse');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var mqttBridge = exports.mqttBridge = function mqttBridge(yomo, spec, iFuncs, compressor) {
  if (!spec) {
    return;
  }
  var client = void 0,
      myId = void 0;

  var online = {};var mySubs = {};var peerSubs = {};
  var qidCounter = 0;
  var nextQid = function nextQid() {
    return (++qidCounter).toString(36);
  };

  var subscribe = function subscribe(peerId, qid, msg, handler) {
    qid = qid || nextQid();
    add2(mySubs, peerId, qid, { msg: msg, handler: handler });
    if (online[peerId]) {
      send(peerId, msg);
    }
    return qid;
  };
  var unsubscribe = function unsubscribe(peerId, qid) {
    del2(mySubs, peerId, qid);
  };

  var psubAutoRun = function psubAutoRun(peerId, qid, fn) {
    return peerSubscribe(peerId, qid, (0, _mobx.autorun)(fn));
  };
  var peerSubscribe = function peerSubscribe(peerId, qid, cb) {
    peerUnSubscribe(peerId, qid);
    add2(peerSubs, peerId, qid, cb);
  };
  var peerUnSubscribe = function peerUnSubscribe(peerId, qid) {
    var cb = del2(peerSubs, peerId, qid);
    if (cb) {
      cb();
    }
  };

  var send = function send(peerId, msg) {
    try {
      //// console.log(`${myId}==>${peerId}`,msg); ////
      msg = JSON.stringify(msg);
      client.publish('data/' + peerId, msg, { qos: 1 });
    } catch (e) {
      console.log('failed send:', peerId);
    }
  };

  var end = function end() {
    client.publish('online/' + myId, '', { retain: true, qos: 1 });
    client.end();
  };

  var startIpc = function startIpc(_ref) {
    var url = _ref.url;
    var id = _ref.id;
    var fmap = _ref.fmap;

    myId = id;var myTopic = 'data/' + myId;
    var will = { topic: 'online/' + myId, payload: '', retain: true, qos: 1 };
    client = _mqtt2.default.connect(url, { will: will });
    client.on('connect', function () {
      client.subscribe('online/#', { qos: 1 });
      client.subscribe(myTopic, { qos: 1 });
      client.publish('online/' + myId, 'ok', { retain: true, qos: 1 });
    });
    client.on('message', function (topic, data) {
      //// console.log(`@${myId} ${topic}: ${data+''}`); ////
      if (topic === myTopic) {
        try {
          data = data.toString();

          var _JSON$parse = JSON.parse(data);

          var _JSON$parse2 = _slicedToArray(_JSON$parse, 3);

          var _peerId = _JSON$parse2[0];
          var qid = _JSON$parse2[1];
          var args = _JSON$parse2[2];

          var _ref2 = (mySubs[_peerId] || {})[qid] || {};

          var handler = _ref2.handler;

          if (handler) {
            handler(args);
          } else {
            console.log('unknown qid ' + _peerId + '==>' + myId + ':', qid, args);
          }
          return;
        } catch (e) {
          console.log('bad message', myId, data, e);
        }
      }
      var peerId = afterPrefix('online/', topic);
      if (peerId) {
        //// console.log('<== online:', peerId, data+''); ////
        if (data.toString() !== '') {
          // start-up: (re)subscribe
          var x = mySubs[peerId] || {};
          for (var _qid in x) {
            var msg = x[_qid].msg;

            if (msg) {
              send(peerId, msg);
            }
          }
          online[peerId] = true;
        } else {
          // shut-down
          var _x = peerSubs[peerId];delete peerSubs[peerId];
          for (var _qid2 in _x) {
            _x[_qid2]();
          }
          delete online[peerId];
        }
      } else {
        console.log('unknown topic:', topic, data + '');
      }
    });
  };

  var proxyFn = function proxyFn(peerId, fname, args, cb) {
    var qid = nextQid();
    var msg = ['!', 'subscribe', [myId, qid, fname, args]];
    subscribe(peerId, qid, msg, cb);
    return function () {
      return send(peerId, ['!', 'unsubscribe', [myId, qid]]);
    };
  };

  var initBridge = function initBridge(_ref3) {
    var url = _ref3.url;
    var id = _ref3.id;
    var fmap = _ref3.fmap;

    subscribe('!', 'subscribe', null, function (_ref4) {
      var _ref5 = _slicedToArray(_ref4, 4);

      var peerId = _ref5[0];
      var qid = _ref5[1];
      var fname = _ref5[2];
      var args = _ref5[3];

      var c = (compressor[fname] || function () {
        return function (x) {
          return x;
        };
      })(args);
      psubAutoRun(peerId, qid, function () {
        try {
          var fn = fmap[fname] || iFuncs[fname];
          if (fn) {
            send(peerId, [myId, qid, c(fn.apply(undefined, [yomo].concat(_toConsumableArray(args))))]);
          } else {
            console.log('unknown fname:', fname);
          }
        } catch (e) {
          console.log('fn exception:', fname, args, e);
        }
      });
    });
    subscribe('!', 'unsubscribe', null, function (_ref6) {
      var _ref7 = _slicedToArray(_ref6, 2);

      var peerId = _ref7[0];
      var qid = _ref7[1];

      peerUnSubscribe(peerId, qid);
      send(peerId, ['!', 'confirmUnsub', [myId, qid]]);
    });
    subscribe('!', 'confirmUnsub', null, function (_ref8) {
      var _ref9 = _slicedToArray(_ref8, 2);

      var peerId = _ref9[0];
      var qid = _ref9[1];

      unsubscribe(peerId, qid);
    });
  };
  startIpc(spec);
  initBridge(spec);

  return { proxyFn: proxyFn, end: end };
};

var add2 = function add2(obj, k1, k2, data) {
  return (obj[k1] = obj[k1] || {})[k2] = data;
};
var del2 = function del2(obj, k1, k2) {
  var x = obj[k1];if (!x) {
    return undefined;
  }
  var v = x[k2];delete x[k2];
  if (emptyObj(x)) {
    delete obj[k1];
  }
  return v;
};
var emptyObj = function emptyObj(obj) {
  for (var i in obj) {
    return false;
  }
  return true;
};
var afterPrefix = function afterPrefix(prefix, str) {
  if (str.substr(0, prefix.length) === prefix) {
    return str.substr(prefix.length);
  } else {
    return undefined;
  }
};