'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mqttIpc = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _mqtt = require('mqtt');

var _mqtt2 = _interopRequireDefault(_mqtt);

var _hash = require('../util/hash2.js');

var _emptyObj = require('../util/emptyObj.js');

var _prefix = require('../util/prefix.js');

var _shortid = require('shortid');

var _shortid2 = _interopRequireDefault(_shortid);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var mqttIpc = exports.mqttIpc = function mqttIpc(ipcSpec, lookup) {
  if (!ipcSpec.myId) {
    ipcSpec = _extends({}, ipcSpec, { myId: 'client/' + _shortid2.default.generate() });
  }
  var _ipcSpec = ipcSpec;
  var ipcUrl = _ipcSpec.ipcUrl;
  var myId = _ipcSpec.myId;


  var online = {}; // Which peers are online?
  var channels = {}; // current channels
  var cmds = {}; // ipc commands
  var defCmd = function defCmd(cmd, handler) {
    return cmds[cmd] = handler;
  };

  var stopAllChannels = function stopAllChannels() {
    (0, _hash.forEach2)(channels, function (channel) {
      return stopChannel(channel);
    });
    for (var k in channels) {
      delete channels[k];
    }
  };

  var qidCounter = 0; // Message channel id:
  var nextQid = function nextQid() {
    return (++qidCounter).toString(36);
  };

  var dataTopic = 'data/' + myId;
  var okTopic = 'online/' + myId;
  var okAll = 'online/#';
  var qos = 1;var retain = true;

  var will = { topic: okTopic, payload: '', retain: retain, qos: qos };
  var client = _mqtt2.default.connect(ipcUrl, { will: will });
  var unsub = function unsub() {
    stopAllChannels();
    client.publish(okTopic, '', { retain: retain, qos: qos }); // like will
    client.end();
  };

  client.on('connect', function () {
    client.subscribe(dataTopic, { qos: qos });
    client.subscribe(okAll, { qos: qos });
    client.publish(okTopic, 'ok', { retain: retain, qos: qos });
  });
  client.on('reconnect', stopAllChannels);
  client.on('close', stopAllChannels);
  client.on('offline', stopAllChannels);
  client.on('error', function (error) {
    return console.log(error);
  });

  var send = function send(peerId, msg) {
    try {
      //// console.log(`${myId}==>${peerId}`,msg); ////
      msg = JSON.stringify(msg);
      client.publish('data/' + peerId, msg, { qos: qos });
    } catch (e) {
      console.log('failed send:', peerId, msg);
    }
  };

  var startChannel = function startChannel(channel) {
    var start = channel.ctrl.start;
    var info = channel.info;

    channel.active = true;
    channel.state = start(info);
    // call info.connect?(args) synchronously!
  };
  var procChannel = function procChannel(channel, data) {
    if (channel) {
      var active = channel.active;
      var proc = channel.ctrl.proc;
      var info = channel.info;
      var state = channel.state;

      if (active) {
        channel.state = proc(info, data, state);
      }
    }
  };
  var stopChannel = function stopChannel(channel) {
    if (channel) {
      var active = channel.active;
      var stop = channel.ctrl.stop;
      var info = channel.info;
      var state = channel.state;

      if (active) {
        stop(info, state);
      }
      channel.state = channel.active = undefined;
    }
  };

  defCmd('data', function (_ref) {
    var _ref2 = _slicedToArray(_ref, 4);

    var __ = _ref2[0];
    var peerId = _ref2[1];
    var qid = _ref2[2];
    var data = _ref2[3];

    procChannel((0, _hash.rd2)(channels, peerId, qid), data);
  });

  // qidS (sending) <----> qidR (receiving)
  // in order to allow two peers to connect both ways
  // without clashing qid sequences, the client adds a
  // star to his qidR (the server's qidS).  So, even when
  // the same qid number is used on both sides, the keys
  // will be different.

  var connectFn = function connectFn(cSpec, recv) {
    var qid = nextQid();
    return connect2(true, cSpec, '*' + qid, qid, recv);
  };
  defCmd('connect', function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 6);

    var __ = _ref4[0];
    var peerId = _ref4[1];
    var qidR = _ref4[2];
    var qidS = _ref4[3];
    var fname = _ref4[4];
    var args = _ref4[5];

    connect2(false, { peerId: peerId, fname: fname, args: args }, qidS, qidR, null);
  });
  var connect2 = function connect2(client, cSpec, qidS, qidR, recv) {
    var peerId = cSpec.peerId;
    var fname = cSpec.fname;
    var args = cSpec.args;

    var server = !client;
    var ok = true;
    lookup(server, cSpec, function (_ref5) {
      var yomo = _ref5.yomo;
      var ctrl = _ref5.ctrl;
      var fnDef = _ref5.fnDef;

      var connect = function connect(args2) {
        return send(peerId, ['connect', myId, qidS, qidR, fname, args2]);
      };
      var sendData = function sendData(data) {
        return send(peerId, ['data', myId, qidS, data]);
      };
      var info = {
        yomo: yomo, fname: fname, args: args, fnDef: fnDef,
        connect: connect, recv: recv, sendData: sendData,
        peerId: peerId, ipcSpec: ipcSpec, client: client, server: server
      };
      var channel = { info: info, ctrl: ctrl };
      ok && (0, _hash.wr2)(channels, peerId, qidR, channel);
      ok && (server || online[peerId]) && startChannel(channel);
    });
    return client && function () {
      ok = false;
      stopChannel((0, _hash.del2)(channels, peerId, qidR));
      send(peerId, ['unsubscribe', myId, qidS]);
    };
  };
  defCmd('unsubscribe', function (_ref6) {
    var _ref7 = _slicedToArray(_ref6, 3);

    var __ = _ref7[0];
    var peerId = _ref7[1];
    var qid = _ref7[2];

    stopChannel((0, _hash.del2)(channels, peerId, qid));
  });

  // Clients initiate connections and reconnect after connection
  // loss.  Servers just forget about dropped connections.

  var peerOnline = function peerOnline(peerId) {
    var x = channels[peerId] || {};
    for (var qid in x) {
      startChannel(x[qid]);
    }
  };
  var peerOffline = function peerOffline(peerId) {
    var x = channels[peerId] || {};
    for (var qid in x) {
      var channel = x[qid];
      stopChannel(channel);
      if (channel.info.server) {
        delete x[qid];
      }
    }
    if ((0, _emptyObj.emptyObj)(x)) {
      delete channels[peerId];
    }
  };

  client.on('message', function (topic, msg) {
    //// console.log(`@${myId} ${topic}: ${msg+''}`); ////
    if (topic === dataTopic) {
      try {
        msg = JSON.parse(msg.toString());
        var handler = cmds[msg[0]];
        if (handler) {
          handler(msg);
        } else {
          console.log('unknown cmd:', msg);
        }
        return;
      } catch (e) {
        return console.log('bad message', myId, msg, e);
      }
    }

    var peerId = (0, _prefix.afterPrefix)('online/', topic);
    if (peerId) {
      if (msg.toString() !== '') {
        // start-up: (re)subscribe
        if (!online[peerId]) {
          // un-bounce ok messages
          online[peerId] = true;
          peerOnline(peerId);
        }
      } else {
        // shut-down
        if (online[peerId]) {
          delete online[peerId];
          peerOffline(peerId);
        }
      }
    } else {
      console.log('unknown topic:', topic, msg + '');
    }
  });

  return { connectFn: connectFn, unsub: unsub };
};