'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pipes = exports.getPipe = exports.linkPipes = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _mobx = require('mobx');

var _cacheFn = require('./cacheFn.js');

var _reuse = require('../util/reuse');

var _hasOwnProperty = require('../util/hasOwnProperty.js');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var startClient = function startClient(info) {
  var yomo = info.yomo;
  var args = info.args;
  var connect = info.connect;
  var done = info.done;

  done && done();info.done = undefined;

  var _args = _slicedToArray(args, 4);

  var type = _args[0];
  var pipeId = _args[1];
  var type2 = _args[2];
  var pipeId2 = _args[3];

  setType(info, type);info.pipeId = pipeId;

  var top1 = (0, _mobx.untracked)(function () {
    return topOf(getPipe(yomo, pipeId));
  });
  connect([type2, pipeId2, top1]);
};
var startSrv = function startSrv(info) {
  var yomo = info.yomo;
  var args = info.args;
  var sendData = info.sendData;

  var _args2 = _slicedToArray(args, 3);

  var type = _args2[0];
  var pipeId = _args2[1];
  var top = _args2[2];

  setType(info, type);info.pipeId = pipeId;

  var top1 = (0, _mobx.untracked)(function () {
    return topOf(getPipe(yomo, pipeId));
  });
  sendData({ top: top1 });

  serve(info, top);
};
var types = {
  data: [true, false],
  pipe: [true, 'bottom'],
  no: [false, false],
  bottom: [false, 'bottom'],
  top: [false, 'top']
};
var setType = function setType(info, type) {
  var _ref = types[type || 'no'] || [];

  var _ref2 = _slicedToArray(_ref, 2);

  info.wantData = _ref2[0];
  info.wantAcc = _ref2[1];
};

var serve = function serve(info, top) {
  var yomo = info.yomo;
  var done = info.done;
  var wantData = info.wantData;
  var wantAcc = info.wantAcc;
  var pipeId = info.pipeId;
  var sendData = info.sendData;

  done && done();
  info.done = (0, _cacheFn.yomoRun)(yomo, false, function () {
    var p = getPipe(yomo, pipeId);

    var msg = void 0;
    if (wantData) {
      var bottom = p.bottom;
      var data = p.data;

      if (top > bottom) {
        data = data.slice(top - bottom);
        bottom = top;
      }
      msg = { bottom: bottom, data: data };
      top = topOf(msg);
    } else {
      msg = {};
    }

    if (wantAcc === 'bottom') {
      msg.acc = p.bottom;
    }
    if (wantAcc === 'top') {
      msg.acc = topOf(p);
    }

    sendData(msg);
  });
};

var proc = function proc(info, data, state) {
  if (data.top !== undefined) {
    return serve(info, data.top);
  }
  var yomo = info.yomo;
  var pipeId = info.pipeId;

  yomo.dispatch(_extends({}, data, { type: 'pipe', id: pipeId }));
};

var stop = function stop(info, state) {
  var done = info.done;

  done && done();info.done = undefined;
};

var linkPipes = exports.linkPipes = {
  client: { start: startClient, proc: proc, stop: stop },
  srv: { start: startSrv, proc: proc, stop: stop }
};

var emptyPipe = { bottom: 0, data: [] };
var topOf = function topOf(_ref3) {
  var bottom = _ref3.bottom;
  var data = _ref3.data;
  return bottom + data.length;
};

var getPipe = exports.getPipe = (0, _cacheFn.cacheFn)(function (yomo, id) {
  return yomo.state().pipes[id] || emptyPipe;
});

var pipes = exports.pipes = function pipes() {
  var state = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var action = arguments[1];

  if (action.type !== 'pipe') {
    return state;
  }
  var id = action.id;
  if (!id) {
    return state;
  }
  return (0, _reuse.reuse)(state, _extends({}, state, _defineProperty({}, id, pipe(state[id], action))));
};

var pipe = function pipe() {
  var state = arguments.length <= 0 || arguments[0] === undefined ? emptyPipe : arguments[0];
  var action = arguments[1];
  var bottom = state.bottom;
  var data = state.data;
  var top = bottom + data.length;
  var b2 = action.bottom;
  var d2 = action.data;
  var value = action.value;
  var acc = action.acc;

  if (d2) {
    if (acc !== undefined && acc > top) {
      bottom = top = acc;data = [];
    }
    if (b2 > top) {
      console.log('skipped data2', state, action);
      data = [].concat(_toConsumableArray(data));data[b2 - bottom - 1] = undefined;top = b2;
    }

    var tmp = b2 === top ? d2 : d2.slice(top - b2);
    if (tmp.length > 0) {
      data = [].concat(_toConsumableArray(data), _toConsumableArray(tmp));
    }
  }
  if ((0, _hasOwnProperty.hasOwnProperty)(action, 'value')) {
    data = [].concat(_toConsumableArray(data), [value]);
  }
  if (acc !== undefined && acc > bottom) {
    data = data.slice(acc - bottom);
    bottom = acc;
  }
  return (0, _reuse.reuse)(state, { bottom: bottom, data: data });
};