'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ProgramErrorIcon = exports.UserErrorIcon = exports.OkIcon = exports.DelayIcon = exports.WaitIcon = exports.yomoRender = exports.ViewException = exports.yomoView = exports.Provider = exports.attachReduxStore = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _withContext = require('recompose/withContext');

var _withContext2 = _interopRequireDefault(_withContext);

var _getContext = require('recompose/getContext');

var _getContext2 = _interopRequireDefault(_getContext);

var _mobxReact = require('mobx-react');

var _newYomo = require('./new-yomo.js');

var _myExceptions = require('../util/my-exceptions.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var attachReduxStore = exports.attachReduxStore = function attachReduxStore(yomo, store) {
  if (yomo.state || yomo.dispatch) {
    console.log('attachReduxStore: Yomo already attached.');
    return;
  }
  var redux = yomo.yomoState('redux');
  var update = function update() {
    return redux.write(store.getState());
  };
  store.subscribe(update);update();
  yomo.initState = function (state) {
    return r.write(state);
  };
  yomo.state = function () {
    return redux.get();
  };
  yomo.dispatch = function (action) {
    return store.dispatch(action);
  };
  yomo.dispatchSoon = function (action) {
    return process.nextTick(yomo.dispatch, action);
  };
};

var Provider = exports.Provider = (0, _withContext2.default)({
  yomo: _react2.default.PropTypes.object,
  store: _react2.default.PropTypes.object
}, function (props) {
  var yomo = props.yomo;
  var store = props.store;

  yomo = yomo || (0, _newYomo.newYomo)();
  store && attachReduxStore(yomo, store);
  return { yomo: yomo, store: store };
})(function (_ref) {
  var children = _ref.children;
  return _react2.default.Children.only(children);
});

var yomoView = exports.yomoView = function yomoView(View, options) {
  return (0, _mobxReact.observer)((0, _getContext2.default)({ yomo: _react2.default.PropTypes.object })(function (props) {
    try {
      return View(props);
    } catch (exception) {
      var waiting = (0, _myExceptions.isWaitX)(exception);
      return _react2.default.createElement(ViewException, { waiting: waiting, exception: exception, options: options });
    }
  }));
};

var ViewException = exports.ViewException = yomoView(function (_ref2) {
  var waiting = _ref2.waiting;
  var exception = _ref2.exception;
  var options = _ref2.options;

  var p = _extends({ size: '1em' }, options);
  if (waiting) {
    return waiting > 0 ? _react2.default.createElement(WaitIcon, p) : _react2.default.createElement(DelayIcon, p);
  } else {
    console.log(exception, exception.stack);
    return _react2.default.createElement(UserErrorIcon, p);
  }
});

var yomoRender = exports.yomoRender = function yomoRender(yomo, View) {
  var domId = arguments.length <= 2 || arguments[2] === undefined ? 'root' : arguments[2];
  return _reactDom2.default.render(_react2.default.createElement(
    Provider,
    { yomo: yomo },
    _react2.default.createElement(View, null)
  ), document.getElementById(domId || 'root'));
};

// -----------------------------------------------------
// icons included here to avoid cyclic module dependency
// -----------------------------------------------------
var css = '.yomoSpin1s {\n  -moz-animation: yomoSpin1s 1s infinite linear;\n  -webkit-animation: yomoSpin1s 1s infinite linear;\n  -ms-animation: yomoSpin1s 1s infinite linear;\n  animation: yomoSpin1s 1s  infinite linear;\n}\n\n@-moz-keyframes yomoSpin1s {\n  0% { -moz-transform: rotate(0deg); }\n  100% { -moz-transform: rotate(360deg); }\n}\n@-webkit-keyframes yomoSpin1s {\n  0% { -webkit-transform: rotate(0deg); }\n  100% { -webkit-transform: rotate(360deg); }\n}\n@keyframes yomoSpin1s {\n  0% {transform:rotate(0deg);}\n  100% {transform:rotate(360deg);}\n}';

var viewBox = '0 0 24 24';
var s0 = '1em';
var strokes = {
  strokeWidth: 2, strokeLinecap: 'round', strokeLinejoin: 'round'
};
var ySpin = 'yomoSpin1s';

var WaitIcon = exports.WaitIcon = yomoView(function (_ref3) {
  var style = _ref3.style;
  var size = _ref3.size;
  return _react2.default.createElement(
    'span',
    null,
    _react2.default.createElement(
      'style',
      { media: 'screen', type: 'text/css' },
      css
    ),
    _react2.default.createElement(
      'svg',
      { style: style, className: ySpin, width: size || s0, viewBox: viewBox },
      _react2.default.createElement(
        'g',
        null,
        _react2.default.createElement('path', _extends({}, strokes, { stroke: '#888888', fill: 'none',
          d: 'M 12 2 a 10 10 0 1 0 10 10'
        }))
      )
    )
  );
});

var DelayIcon = exports.DelayIcon = yomoView(function (_ref4) {
  var style = _ref4.style;
  var size = _ref4.size;
  return _react2.default.createElement(
    'svg',
    { style: style, width: size || s0, viewBox: viewBox },
    _react2.default.createElement(
      'g',
      null,
      _react2.default.createElement('path', _extends({}, strokes, { stroke: '#888888', fill: 'none',
        d: 'M 12 2 a 10 10 0 1 0 10 10'
      }))
    )
  );
});
var OkIcon = exports.OkIcon = yomoView(function (_ref5) {
  var style = _ref5.style;
  var size = _ref5.size;
  return _react2.default.createElement(
    'svg',
    { style: style, width: size || s0, viewBox: viewBox },
    _react2.default.createElement(
      'g',
      null,
      _react2.default.createElement('circle', { cx: 12, cy: 12, r: 12, fill: 'green' }),
      _react2.default.createElement('path', _extends({}, strokes, { stroke: '#FFFFFF', fill: 'none',
        d: 'M6,13 l4,4 l8.5,-8.5'
      }))
    )
  );
});
var UserErrorIcon = exports.UserErrorIcon = yomoView(function (_ref6) {
  var style = _ref6.style;
  var size = _ref6.size;
  return _react2.default.createElement(
    'svg',
    { style: style, width: size || s0, viewBox: viewBox },
    _react2.default.createElement(
      'g',
      null,
      _react2.default.createElement('path', _extends({}, strokes, { stroke: '#D80000', fill: '#D80000',
        d: 'M1,23 l22,0 l-11,-22 z'
      })),
      _react2.default.createElement('path', _extends({}, strokes, { stroke: '#FFFFFF',
        d: 'M12,8 l0,7'
      })),
      _react2.default.createElement('path', _extends({}, _extends({}, strokes, { strokeWidth: 3 }), { stroke: '#FFFFFF',
        d: 'M12,19.5 l0,0'
      }))
    )
  );
});
var ProgramErrorIcon = exports.ProgramErrorIcon = yomoView(function (_ref7) {
  var style = _ref7.style;
  var size = _ref7.size;
  return _react2.default.createElement(
    'svg',
    { style: style, width: size || s0, viewBox: viewBox },
    _react2.default.createElement(
      'g',
      null,
      _react2.default.createElement('path', _extends({}, strokes, { stroke: '#AA0000', fill: '#AA0000',
        d: 'M1,23 l22,0 l-11,-22 z'
      })),
      _react2.default.createElement('path', _extends({}, strokes, { stroke: '#FFFF00',
        d: 'M12,8 l0,7'
      })),
      _react2.default.createElement('path', _extends({}, _extends({}, strokes, { strokeWidth: 3 }), { stroke: '#FFFF00',
        d: 'M12,19.5 l0,0'
      }))
    )
  );
});